@@ -1,7 +1,7 @@
 package Perinci::CmdLine::Base;
 
 our $DATE = '2015-03-18'; # DATE
-our $VERSION = '0.95'; # VERSION
+our $VERSION = '0.96'; # VERSION
 
 use 5.010001;
 use strict;
@@ -704,13 +704,13 @@ sub _parse_argv2 {
             }
             # this will probably be eventually checked by the rinci function's
             # schema: stream arguments need to have cmdline_src set to
-            # stdin_or_files, stdin, or file.
+            # stdin_or_file, stdin_or_files, stdin, or file.
             if ($av->{stream}) {
                 unless ($av->{cmdline_src} &&
                             $av->{cmdline_src} =~
-                                /\A(stdin|file|stdin_or_files)\z/) {
+                                /\A(stdin|file|stdin_or_files?)\z/) {
                     die "BUG: stream argument '$ak' needs to have cmdline_src ".
-                        "set to stdin, file, or stdin_or_files";
+                        "set to stdin, file, stdin_or_file, or stdin_or_files";
                 }
             }
         }
@@ -840,8 +840,8 @@ sub parse_cmdline_src {
             (
                 !$csa || !$csb ? 0 :
                     $csa eq 'stdin_line' && $csb eq 'stdin_line' ? 0 :
-                    $csa eq 'stdin_line' && $csb =~ /^(stdin|stdin_or_files)/ ? -1 :
-                    $csb eq 'stdin_line' && $csa =~ /^(stdin|stdin_or_files)/ ? 1 : 0
+                    $csa eq 'stdin_line' && $csb =~ /^(stdin|stdin_or_files?)/ ? -1 :
+                    $csb eq 'stdin_line' && $csa =~ /^(stdin|stdin_or_files?)/ ? 1 : 0
             )
             ||
 
@@ -862,14 +862,14 @@ sub parse_cmdline_src {
             if ($src) {
                 die [531,
                      "Invalid 'cmdline_src' value for argument '$an': $src"]
-                    unless $src =~ /\A(stdin|file|stdin_or_files|stdin_line)\z/;
+                    unless $src =~ /\A(stdin|file|stdin_or_files?|stdin_line)\z/;
                 die [531,
                      "Sorry, argument '$an' is set cmdline_src=$src, but type ".
                          "is not str/buf/array, only those are supported now"]
                     unless $do_stream || $type =~ /\A(str|buf|array)\z/;
-                if ($src =~ /\A(stdin|stdin_or_files)\z/) {
+                if ($src =~ /\A(stdin|stdin_or_files?)\z/) {
                     die [531, "Only one argument can be specified ".
-                             "cmdline_src stdin/stdin_or_files"]
+                             "cmdline_src stdin/stdin_or_file/stdin_or_files"]
                         if $stdin_seen++;
                 }
                 my $is_ary = $type eq 'array';
@@ -898,14 +898,19 @@ sub parse_cmdline_src {
                             $is_ary ? [<STDIN>] :
                                 do {local $/; ~~<STDIN>};
                     $r->{args}{"-cmdline_src_$an"} = 'stdin';
-                } elsif ($src eq 'stdin_or_files') {
+                } elsif ($src eq 'stdin_or_file' || $src eq 'stdin_or_files') {
                     # push back argument value to @ARGV so <> can work to slurp
                     # all the specified files
                     local @ARGV = @ARGV;
                     unshift @ARGV, $r->{args}{$an}
                         if defined $r->{args}{$an};
+
+                    # with stdin_or_file, we only accept one file
+                    splice @ARGV, 1
+                        if @ARGV > 1 && $src eq 'stdin_or_file';
+
                     #$log->tracef("Getting argument '$an' value from ".
-                    #                 "stdin_or_files, \@ARGV=%s ...", \@ARGV);
+                    #                 "$src, \@ARGV=%s ...", \@ARGV);
 
                     # perl doesn't seem to check files, so we check it here
                     for (@ARGV) {
@@ -918,7 +923,7 @@ sub parse_cmdline_src {
                         __gen_iter(\*ARGV, $as->{schema}, $an) :
                             $is_ary ? [<>] :
                                 do {local $/; ~~<>};
-                    $r->{args}{"-cmdline_src_$an"} = 'stdin_or_files';
+                    $r->{args}{"-cmdline_src_$an"} = $src;
                 } elsif ($src eq 'file') {
                     unless (exists $r->{args}{$an}) {
                         if ($as->{req}) {
@@ -943,7 +948,7 @@ sub parse_cmdline_src {
                             $is_ary ? [<$fh>] :
                                 do { local $/; ~~<$fh> };
                     $r->{args}{"-cmdline_src_$an"} = 'file';
-                    $r->{args}{"-cmdline_srcfilename_$an"} = $fname;
+                    $r->{args}{"-cmdline_srcfilenames_$an"} = [$fname];
                 }
             }
 
@@ -1163,7 +1168,7 @@ Perinci::CmdLine::Base - Base class for Perinci::CmdLine{Classic,::Lite}
 
 =head1 VERSION
 
-This document describes version 0.95 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-03-18.
+This document describes version 0.96 of Perinci::CmdLine::Base (from Perl distribution Perinci-CmdLine-Lite), released on 2015-03-18.
 
 =head1 DESCRIPTION
 
@@ -1849,15 +1854,21 @@ Only when in dry run mode, to notify function that we are in dry run mode.
 Only when C<pass_cmdline_object> attribute is set to true. This can be useful
 for the function to know about various stuffs, by probing the framework object.
 
+=head2 -cmdline_r => hash
+
+Only when C<pass_cmdline_object> attribute is set to true. Contains the C<$r>
+per-request hash/stash. This can be useful for the function to know about
+various stuffs, e.g. parsed configuration data, etc.
+
 =head2 -cmdline_src_ARGNAME => str
 
 This will be set if argument is retrieved from C<file>, C<stdin>,
-C<stdin_or_files>, or C<stdin_line>.
+C<stdin_or_file>, C<stdin_or_files>, or C<stdin_line>.
 
-=head2 -cmdline_srcfilename_ARGNAME => str
+=head2 -cmdline_srcfilenames_ARGNAME => array
 
-An extra information if argument value is retrieved from C<file>, so the
-function can know the filename.
+An extra information if argument value is retrieved from file(s), so the
+function can know the filename(s).
 
 =head1 METADATA PROPERTY ATTRIBUTE
 
